/////////////////////////////////////////////////////////////
//                                                         //
//               Nokia 8310 LCD driver v1.0                //
//                   (OM6211 controller)                   //
//               by Oroszfji Szabolcs 2008                //
//  (special thanks to Rifer for initial PCD8544 project)  //                                     //
//                                                         //
//            ..: LCDHype by ViRuSTriNiTy :..              //
//                                                         //
/////////////////////////////////////////////////////////////

unit Controller_OM6211;

interface
uses
  SysUtils, Windows, inifiles;

const ID = 'Nokia 8310   v1.0   (OM6211)'; //ID sets the description of the dll
      MAX_RESPIXELS = 4032; // 84*48

Const CharTableDefault : array [0..479] of byte = ( $00,$00,$00,$00,$00,        // 20 SP
                                                    $00,$00,$5f,$00,$00,        // 21 !
{Standard char table}                               $00,$07,$00,$07,$00,        // 22 "
                                                    $14,$7f,$14,$7f,$14,        // 23 #
                                                    $24,$2a,$7f,$2a,$12,        // 24 $
                                                    $23,$13,$08,$64,$62,        // 25 %
                                                    $36,$49,$55,$22,$50,        // 26 &
                                                    $00,$05,$03,$00,$00,        // 27 '
                                                    $00,$1c,$22,$41,$00,        // 28 (
                                                    $00,$41,$22,$1c,$00,        // 29 )
                                                    $14,$08,$3e,$08,$14,        // 2a *
                                                    $08,$08,$3e,$08,$08,        // 2b +
                                                    $00,$50,$30,$00,$00,        // 2c ,
                                                    $08,$08,$08,$08,$08,        // 2d -
                                                    $00,$60,$60,$00,$00,        // 2e .
                                                    $20,$10,$08,$04,$02,        // 2f /
                                                    $3e,$51,$49,$45,$3e,        // 30 0
                                                    $00,$42,$7f,$40,$00,        // 31 1
                                                    $42,$61,$51,$49,$46,        // 32 2
                                                    $21,$41,$45,$4b,$31,        // 33 3
                                                    $18,$14,$12,$7f,$10,        // 34 4
                                                    $27,$45,$45,$45,$39,        // 35 5
                                                    $3c,$4a,$49,$49,$30,        // 36 6
                                                    $01,$71,$09,$05,$03,        // 37 7
                                                    $36,$49,$49,$49,$36,        // 38 8
                                                    $06,$49,$49,$29,$1e,        // 39 9
                                                    $00,$36,$36,$00,$00,        // 3a :
                                                    $00,$56,$36,$00,$00,        // 3b ;
                                                    $08,$14,$22,$41,$00,        // 3c <
                                                    $14,$14,$14,$14,$14,        // 3d =
                                                    $00,$41,$22,$14,$08,        // 3e >
                                                    $02,$01,$51,$09,$06,        // 3f ?
                                                    $32,$49,$79,$41,$3e,        // 40 @
                                                    $7e,$11,$11,$11,$7e,        // 41 A
                                                    $7f,$49,$49,$49,$36,        // 42 B
                                                    $3e,$41,$41,$41,$22,        // 43 C
                                                    $7f,$41,$41,$22,$1c,        // 44 D
                                                    $7f,$49,$49,$49,$41,        // 45 E
                                                    $7f,$09,$09,$09,$01,        // 46 F
                                                    $3e,$41,$49,$49,$7a,        // 47 G
                                                    $7f,$08,$08,$08,$7f,        // 48 H
                                                    $00,$41,$7f,$41,$00,        // 49 I
                                                    $20,$40,$41,$3f,$01,        // 4a J
                                                    $7f,$08,$14,$22,$41,        // 4b K
                                                    $7f,$40,$40,$40,$40,        // 4c L
                                                    $7f,$02,$0c,$02,$7f,        // 4d M
                                                    $7f,$04,$08,$10,$7f,        // 4e N
                                                    $3e,$41,$41,$41,$3e,        // 4f O
                                                    $7f,$09,$09,$09,$06,        // 50 P
                                                    $3e,$41,$51,$21,$5e,        // 51 Q
                                                    $7f,$09,$19,$29,$46,        // 52 R
                                                    $46,$49,$49,$49,$31,        // 53 S
                                                    $01,$01,$7f,$01,$01,        // 54 T
                                                    $3f,$40,$40,$40,$3f,        // 55 U
                                                    $1f,$20,$40,$20,$1f,        // 56 V
                                                    $3f,$40,$38,$40,$3f,        // 57 W
                                                    $63,$14,$08,$14,$63,        // 58 X
                                                    $07,$08,$70,$08,$07,        // 59 Y
                                                    $61,$51,$49,$45,$43,        // 5a Z
                                                    $00,$7f,$41,$41,$00,        // 5b [
                                                    $02,$04,$08,$10,$20,        // 5c \
                                                    $00,$41,$41,$7f,$00,        // 5d ]
                                                    $04,$02,$01,$02,$04,        // 5e ^
                                                    $40,$40,$40,$40,$40,        // 5f _
                                                    $00,$01,$02,$04,$00,        // 60 '
                                                    $20,$54,$54,$54,$78,        // 61 a
                                                    $7f,$48,$44,$44,$38,        // 62 b
                                                    $38,$44,$44,$44,$20,        // 63 c
                                                    $38,$44,$44,$48,$7f,        // 64 d
                                                    $38,$54,$54,$54,$18,        // 65 e
                                                    $08,$7e,$09,$01,$02,        // 66 f
                                                    $0c,$52,$52,$52,$3e,        // 67 g
                                                    $7f,$08,$04,$04,$78,        // 68 h
                                                    $00,$44,$7d,$40,$00,        // 69 i
                                                    $20,$40,$44,$3d,$00,        // 6a j
                                                    $7f,$10,$28,$44,$00,        // 6b k
                                                    $00,$41,$7f,$40,$00,        // 6c l
                                                    $7c,$04,$18,$04,$78,        // 6d m
                                                    $7c,$08,$04,$04,$78,        // 6e n
                                                    $38,$44,$44,$44,$38,        // 6f o
                                                    $7c,$14,$14,$14,$08,        // 70 p
                                                    $08,$14,$14,$18,$7c,        // 71 q
                                                    $7c,$08,$04,$04,$08,        // 72 r
                                                    $48,$54,$54,$54,$20,        // 73 s
                                                    $04,$3f,$44,$40,$20,        // 74 t
                                                    $3c,$40,$40,$20,$7c,        // 75 u
                                                    $1c,$20,$40,$20,$1c,        // 76 v
                                                    $3c,$40,$30,$40,$3c,        // 77 w
                                                    $44,$28,$10,$28,$44,        // 78 x
                                                    $0c,$50,$50,$50,$3c,        // 79 y
                                                    $44,$64,$54,$4c,$44,        // 7a z
                                                    $00,$08,$36,$41,$00,        // 7b {
                                                    $00,$00,$7f,$00,$00,        // 7c 
                                                    $00,$41,$36,$08,$00,        // 7d }
                                                    $10,$08,$08,$10,$08,        // 7e ~
                                                    $78,$46,$41,$46,$78 );      // 7f

Const CharTableCyrillic : array [0..479] of byte = ($00,$00,$00,$00,$00,        // 20 SP
                                                    $00,$00,$5f,$00,$00,        // 21 !
{Cyrillic char table - not implemented yet}         $00,$07,$00,$07,$00,        // 22 "
                                                    $14,$7f,$14,$7f,$14,        // 23 #
                                                    $24,$2a,$7f,$2a,$12,        // 24 $
                                                    $23,$13,$08,$64,$62,        // 25 %
                                                    $36,$49,$55,$22,$50,        // 26 &
                                                    $00,$05,$03,$00,$00,        // 27 '
                                                    $00,$1c,$22,$41,$00,        // 28 (
                                                    $00,$41,$22,$1c,$00,        // 29 )
                                                    $14,$08,$3e,$08,$14,        // 2a *
                                                    $08,$08,$3e,$08,$08,        // 2b +
                                                    $00,$50,$30,$00,$00,        // 2c ,
                                                    $08,$08,$08,$08,$08,        // 2d -
                                                    $00,$60,$60,$00,$00,        // 2e .
                                                    $20,$10,$08,$04,$02,        // 2f /
                                                    $3e,$51,$49,$45,$3e,        // 30 0
                                                    $00,$42,$7f,$40,$00,        // 31 1
                                                    $42,$61,$51,$49,$46,        // 32 2
                                                    $21,$41,$45,$4b,$31,        // 33 3
                                                    $18,$14,$12,$7f,$10,        // 34 4
                                                    $27,$45,$45,$45,$39,        // 35 5
                                                    $3c,$4a,$49,$49,$30,        // 36 6
                                                    $01,$71,$09,$05,$03,        // 37 7
                                                    $36,$49,$49,$49,$36,        // 38 8
                                                    $06,$49,$49,$29,$1e,        // 39 9
                                                    $00,$36,$36,$00,$00,        // 3a :
                                                    $00,$56,$36,$00,$00,        // 3b ;
                                                    $08,$14,$22,$41,$00,        // 3c <
                                                    $14,$14,$14,$14,$14,        // 3d =
                                                    $00,$41,$22,$14,$08,        // 3e >
                                                    $02,$01,$51,$09,$06,        // 3f ?
                                                    $32,$49,$79,$41,$3e,        // 40 @
                                                    $7e,$11,$11,$11,$7e,        // 41 A
                                                    $7f,$49,$49,$49,$36,        // 42 B
                                                    $3e,$41,$41,$41,$22,        // 43 C
                                                    $7f,$41,$41,$22,$1c,        // 44 D
                                                    $7f,$49,$49,$49,$41,        // 45 E
                                                    $7f,$09,$09,$09,$01,        // 46 F
                                                    $3e,$41,$49,$49,$7a,        // 47 G
                                                    $7f,$08,$08,$08,$7f,        // 48 H
                                                    $00,$41,$7f,$41,$00,        // 49 I
                                                    $20,$40,$41,$3f,$01,        // 4a J
                                                    $7f,$08,$14,$22,$41,        // 4b K
                                                    $7f,$40,$40,$40,$40,        // 4c L
                                                    $7f,$02,$0c,$02,$7f,        // 4d M
                                                    $7f,$04,$08,$10,$7f,        // 4e N
                                                    $3e,$41,$41,$41,$3e,        // 4f O
                                                    $7f,$09,$09,$09,$06,        // 50 P
                                                    $3e,$41,$51,$21,$5e,        // 51 Q
                                                    $7f,$09,$19,$29,$46,        // 52 R
                                                    $46,$49,$49,$49,$31,        // 53 S
                                                    $01,$01,$7f,$01,$01,        // 54 T
                                                    $3f,$40,$40,$40,$3f,        // 55 U
                                                    $1f,$20,$40,$20,$1f,        // 56 V
                                                    $3f,$40,$38,$40,$3f,        // 57 W
                                                    $63,$14,$08,$14,$63,        // 58 X
                                                    $07,$08,$70,$08,$07,        // 59 Y
                                                    $61,$51,$49,$45,$43,        // 5a Z
                                                    $00,$7f,$41,$41,$00,        // 5b [
                                                    $02,$04,$08,$10,$20,        // 5c \
                                                    $00,$41,$41,$7f,$00,        // 5d ]
                                                    $04,$02,$01,$02,$04,        // 5e ^
                                                    $40,$40,$40,$40,$40,        // 5f _
                                                    $00,$01,$02,$04,$00,        // 60 '
                                                    $20,$54,$54,$54,$78,        // 61 a
                                                    $7f,$48,$44,$44,$38,        // 62 b
                                                    $38,$44,$44,$44,$20,        // 63 c
                                                    $38,$44,$44,$48,$7f,        // 64 d
                                                    $38,$54,$54,$54,$18,        // 65 e
                                                    $08,$7e,$09,$01,$02,        // 66 f
                                                    $0c,$52,$52,$52,$3e,        // 67 g
                                                    $7f,$08,$04,$04,$78,        // 68 h
                                                    $00,$44,$7d,$40,$00,        // 69 i
                                                    $20,$40,$44,$3d,$00,        // 6a j
                                                    $7f,$10,$28,$44,$00,        // 6b k
                                                    $00,$41,$7f,$40,$00,        // 6c l
                                                    $7c,$04,$18,$04,$78,        // 6d m
                                                    $7c,$08,$04,$04,$78,        // 6e n
                                                    $38,$44,$44,$44,$38,        // 6f o
                                                    $7c,$14,$14,$14,$08,        // 70 p
                                                    $08,$14,$14,$18,$7c,        // 71 q
                                                    $7c,$08,$04,$04,$08,        // 72 r
                                                    $48,$54,$54,$54,$20,        // 73 s
                                                    $04,$3f,$44,$40,$20,        // 74 t
                                                    $3c,$40,$40,$20,$7c,        // 75 u
                                                    $1c,$20,$40,$20,$1c,        // 76 v
                                                    $3c,$40,$30,$40,$3c,        // 77 w
                                                    $44,$28,$10,$28,$44,        // 78 x
                                                    $0c,$50,$50,$50,$3c,        // 79 y
                                                    $44,$64,$54,$4c,$44,        // 7a z
                                                    $00,$08,$36,$41,$00,        // 7b {
                                                    $00,$00,$7f,$00,$00,        // 7c 
                                                    $00,$41,$36,$08,$00,        // 7d }
                                                    $10,$08,$08,$10,$08,        // 7e ~
                                                    $78,$46,$41,$46,$78 );      // 7f

Const CharTableSpecial : array [0..639] of byte = ( $00,$00,$00,$00,$00,
                                                    $00,$00,$00,$00,$00,
{Hungarian, German, Romanian special char table}    $00,$00,$00,$00,$00,
                                                    $00,$00,$00,$00,$00,
                                                    $00,$00,$00,$00,$00,
                                                    $00,$00,$00,$00,$00,
                                                    $00,$00,$00,$00,$00,
                                                    $00,$00,$00,$00,$00,
                                                    $00,$00,$00,$00,$00,
                                                    $00,$00,$00,$00,$00,
                                                    $00,$00,$00,$00,$00,
                                                    $00,$00,$00,$00,$00,
                                                    $00,$00,$00,$00,$00,
                                                    $00,$00,$00,$00,$00,
                                                    $00,$00,$00,$00,$00,
                                                    $00,$00,$00,$00,$00,
                                                    $00,$00,$00,$00,$00,
                                                    $00,$00,$00,$00,$00,
                                                    $00,$00,$00,$00,$00,
                                                    $00,$00,$00,$00,$00,
                                                    $00,$00,$00,$00,$00,
                                                    $00,$00,$00,$00,$00,
                                                    $00,$00,$00,$00,$00,
                                                    $00,$00,$00,$00,$00,
                                                    $00,$00,$00,$00,$00,
                                                    $00,$00,$00,$00,$00,
                                                    $00,$00,$00,$00,$00,
                                                    $00,$00,$00,$00,$00,
                                                    $00,$00,$00,$00,$00,
                                                    $00,$00,$00,$00,$00,
                                                    $00,$00,$00,$00,$00,
                                                    $00,$00,$00,$00,$00,
                                                    $00,$00,$00,$00,$00,
                                                    $00,$00,$00,$00,$00,
                                                    $00,$00,$00,$00,$00,
                                                    $00,$00,$00,$00,$00,
                                                    $00,$00,$00,$00,$00,
                                                    $00,$00,$00,$00,$00,
                                                    $00,$00,$00,$00,$00,
                                                    $00,$00,$00,$00,$00,
                                                    $00,$00,$00,$00,$00,
                                                    $00,$00,$00,$00,$00,
                                                    $46,$49,$C9,$49,$31,        //  ROM S
                                                    $00,$00,$00,$00,$00,
                                                    $00,$00,$00,$00,$00,
                                                    $00,$00,$00,$00,$00,
                                                    $00,$00,$00,$00,$00,
                                                    $00,$00,$00,$00,$00,
                                                    $00,$00,$00,$00,$00,
                                                    $00,$00,$00,$00,$00,
                                                    $00,$00,$00,$00,$00,
                                                    $00,$00,$00,$00,$00,
                                                    $00,$00,$00,$00,$00,
                                                    $00,$00,$00,$00,$00,
                                                    $00,$00,$00,$00,$00,
                                                    $00,$00,$00,$00,$00,
                                                    $00,$00,$00,$00,$00,
                                                    $00,$00,$00,$00,$00,
                                                    $48,$54,$D4,$54,$20,        //   ROM s
                                                    $00,$00,$00,$00,$00,
                                                    $00,$00,$00,$00,$00,
                                                    $00,$00,$00,$00,$00,
                                                    $00,$00,$00,$00,$00,
                                                    $00,$00,$00,$00,$00,
                                                    $00,$00,$00,$00,$00,
                                                    $F0,$28,$26,$29,$F0,        //   HUN A
                                                    $F0,$2A,$29,$2A,$F0,        //   ROM A1
                                                    $F0,$29,$2A,$29,$F0,        //   ROM A2
                                                    $F0,$29,$24,$29,$F0,        //   GER A
                                                    $00,$00,$00,$00,$00,
                                                    $00,$00,$00,$00,$00,
                                                    $00,$00,$00,$00,$00,
                                                    $00,$00,$00,$00,$00,
                                                    $F8,$A8,$AA,$A9,$88,        //   HUN E
                                                    $00,$00,$00,$00,$00,
                                                    $00,$00,$00,$00,$00,
                                                    $00,$00,$00,$00,$00,
                                                    $00,$88,$FA,$89,$00,        //   HUN I
                                                    $00,$8A,$F9,$8A,$00,        //   ROM I
                                                    $00,$00,$00,$00,$00,
                                                    $00,$00,$00,$00,$00,
                                                    $00,$00,$00,$00,$00,
                                                    $00,$00,$00,$00,$00,
                                                    $78,$84,$86,$85,$78,        //   HUN O1
                                                    $00,$00,$00,$00,$00,
                                                    $78,$86,$85,$86,$79,        //   HUN O2
                                                    $78,$85,$84,$85,$78,        //   HUN/GER O3
                                                    $00,$00,$00,$00,$00,
                                                    $00,$00,$00,$00,$00,
                                                    $00,$00,$00,$00,$00,
                                                    $7C,$80,$82,$81,$7C,        //   HUN U1
                                                    $7C,$82,$81,$82,$7D,        //   HUN U2
                                                    $7C,$81,$80,$81,$7C,        //   HUN/GER U3
                                                    $00,$00,$00,$00,$00,
                                                    $01,$81,$7F,$01,$01,        //   ROM T
                                                    $80,$7C,$92,$92,$6C,        //   GER S
                                                    $00,$00,$00,$00,$00,
                                                    $40,$A8,$AA,$A9,$F0,        //   HUN a
                                                    $40,$AA,$A9,$AA,$F0,        //   ROM a1
                                                    $40,$A9,$AA,$A9,$F0,        //   ROM a2
                                                    $40,$AA,$A8,$AA,$F0,        //   GER a
                                                    $00,$00,$00,$00,$00,
                                                    $00,$00,$00,$00,$00,
                                                    $00,$00,$00,$00,$00,
                                                    $00,$00,$00,$00,$00,
                                                    $70,$A8,$AA,$A9,$30,        //   HUN e
                                                    $00,$00,$00,$00,$00,
                                                    $70,$AA,$A8,$AA,$30,        //   e
                                                    $00,$00,$00,$00,$00,
                                                    $00,$90,$FA,$81,$00,        //   HUN i
                                                    $00,$92,$F9,$82,$00,        //   ROM i
                                                    $00,$00,$00,$00,$00,
                                                    $00,$00,$00,$00,$00,
                                                    $00,$00,$00,$00,$00,
                                                    $00,$00,$00,$00,$00,
                                                    $70,$88,$8A,$89,$70,        //   HUN o1
                                                    $70,$8A,$89,$8A,$71,        //   HUN o2
                                                    $70,$8A,$89,$8A,$71,        //   HUN o2
                                                    $70,$8A,$88,$8A,$70,        //   HUN/GER o3
                                                    $00,$00,$00,$00,$00,
                                                    $00,$00,$00,$00,$00,
                                                    $00,$00,$00,$00,$00,
                                                    $78,$80,$82,$41,$F8,        //   HUN u1
                                                    $78,$82,$81,$42,$F9,        //   HUN u2
                                                    $78,$82,$80,$42,$F8,        //   HUN/GER u3
                                                    $00,$00,$00,$00,$00,
                                                    $04,$3F,$C4,$40,$20,        //   ROM t
                                                    $00,$00,$00,$00,$00 );

Type TArrayType = array[0..MAX_RESPIXELS-1] of byte;

type TCharData = record
                  position : byte;
                  data : array[0..7,0..7] of byte;
                 End;
     TDLLInfo = record
                 IDArray : array[0..255] of byte; //holds identifier of DLL
                 SupportGfxLCD : Boolean;         //does this driver support graphical LCDs?
                 SupportTxtLCD : Boolean;         //does this driver support text
                 SupportLightSlider : Boolean;    //does this driver support the light control slider
                 SupportContrastSlider : Boolean; //does this driver support the contrast slider
                 SupportOutports : Boolean;       //does this driver support outports for controlling external circuits
                 CCharWidth : Byte;               //custom char width in pixels
                 CCharHeight : Byte;              //custom char height in pixels
                 FontPitch : Byte;                //fontpitch of LCD in pixels
                End;
     TSymbolInfo = record
                    SymbolData : array[0..19] of boolean;
                   End;

var
  Ini : Tinifile;
  Light : INTEGER;
  Once : boolean = true; // used to display BootLogo
  LCD_Width : integer = 84;
  LCD_Height : integer = 48;
  LCD_TextCols : integer = 14;
  LCD_TextRows : integer = 6;
  LCD_PosX : Integer = 0;
  LCD_PosY : Integer = 0;
  Font_Height : integer = 8;
  ScanSize : integer = 6; // 48/8
  LCD_Port : integer = $378;
  LCD_ExectimeGfx : integer = 0;
  LCD_Exectime : integer = 0;
  ContrastValue : byte;
  Contrast_ON : boolean;
  GraphArray : array[0..83,0..5] of byte; // max 84x6  holds the temporary graphic data
  TextArray : array[0..83,0..5] of byte; // max 84x6   holds the temporary text data
  CGchar : array[0..31*6] of byte; // 32 'CGRAM' characters  holds the CGRAM data
  FontType : string;
  CommandModeSet : string;

Procedure Logo84x48;stdcall; forward
//Sends Data to the Controller Chip
procedure LCD_SendToController(Value : Byte); stdcall;

//Sends Data to the integrated LCD Memory
//on adress specified by LCD_SetOutputAdress
procedure LCD_SendToMemory(Value : Byte); stdcall;

//Sends Data to LCD Gfx Memory
procedure LCD_SendToGfxMemory(Pixels : TArrayType; X1,Y1,X2,Y2 : Word; inverted : boolean); stdcall;

//Sets the LCD RAM Adress for Outputs
procedure LCD_SetOutputAddress(x,y : Cardinal); stdcall;

//Sets a LCD Custom Char specified in cdata
procedure LCD_SetCGRAMChar(cdata : TCharData); stdcall;

//Returns the Custom Char bytecode for the current display
//Based on the HD44780 eg. the input for the first Custom Char
//is position=0 and the return value is the equivalent bytecode
//to the HD44780 Custom Char bytecode
function LCD_GetCGRAMChar(position : Byte) : Byte; stdcall;

//Initialize the LCD
procedure LCD_Init; stdcall;

//Called when DLL released
procedure LCD_CleanUp; stdcall;

//Sets the Port Adress where the LCD are connected and
//the lcd execution time for processing sended data
procedure LCD_SetIOPropertys(Port : pchar;
                             Exectime : Cardinal;
                             ExectimeGfx : Cardinal;
                             X : Cardinal;
                             Y : Cardinal;
                             gX : Cardinal;
                             gY : Cardinal;
                             LightOn : Boolean;
                             LightSliderValue : Byte;
                             ContrastOn : Boolean;
                             ContrastSliderValue : Byte;
                             Outports : Cardinal;
                             UnderlineMode : Boolean;
                             UnderlineOutput : Boolean); stdcall;

//This is a routine for customizing output,
//by default only used by m50530 symbol lcd
procedure LCD_Customize(SymData : TSymbolInfo); stdcall;

//Caller of this function gets some info about the DLL
function DLL_GetInfo : TDLLInfo; stdcall;

//Signals if the write sequence to LCD is ended or not
function LCD_IsReadyToReceive : boolean; stdcall;


implementation

//Imported DLL Routine from DriverLinx PortIO DLL Driver
function DlPortReadPortUchar(Port:longint):byte;stdcall;external 'DLPORTIO.DLL';
procedure DlPortWritePortUchar(Port:longint;Value:byte);stdcall;external 'DLPORTIO.DLL';

{-------------------------------------------------------}

Procedure Wait(x : Integer);

var c : Integer;
Begin
     for c:=0 to x do  ;
      DlPortReadPortUchar($220); //read = 1 microsecond delay
End;

{-------------------------------------------------------}

Procedure Command (Value: Byte; Exectime : Cardinal);

Var i : Byte;
    j : integer;
Begin
DlPortWritePortUchar(LCD_Port,14+Light);

DlPortWritePortUchar(LCD_Port+2,3); // command=0
  for j := 1 to Exectime do  DLPortReadPortUChar($220);
DlPortWritePortUchar(LCD_Port+2,1);
  for j := 1 to Exectime do  DLPortReadPortUChar($220);

For i := 7 downto 0 do
if (Value AND (1 shl i))>0 then
  Begin
   try
    DlPortWritePortUchar(LCD_Port+2,2);
      for j := 1 to Exectime do  DLPortReadPortUChar($220);
    DlPortWritePortUchar(LCD_Port+2,0);
      for j := 1 to Exectime do  DLPortReadPortUChar($220);
   except else End
  End
else
  Begin
   try
    DlPortWritePortUchar(LCD_Port+2,3);
      for j := 1 to Exectime do  DLPortReadPortUChar($220);
    DlPortWritePortUchar(LCD_Port+2,1);
      for j := 1 to Exectime do  DLPortReadPortUChar($220);
   except else End
  End;
End;

{-------------------------------------------------------}

Procedure Data (Value: Byte; Exectime : Cardinal);

Var i : Byte;
    j : integer;
Begin
DlPortWritePortUchar(LCD_Port,14+Light);

DlPortWritePortUchar(LCD_Port+2,2);  // data=1
  for j := 1 to Exectime do  DLPortReadPortUChar($220);
DlPortWritePortUchar(LCD_Port+2,0);
  for j := 1 to Exectime do  DLPortReadPortUChar($220);

For i := 7 downto 0 do
if (Value AND (1 shl i))>0 then
  Begin
   try
    DlPortWritePortUchar(LCD_Port+2,2);
      for j := 1 to Exectime do  DLPortReadPortUChar($220);
    DlPortWritePortUchar(LCD_Port+2,0);
      for j := 1 to Exectime do  DLPortReadPortUChar($220);
   except else End
  End
else
  Begin
   try
    DlPortWritePortUchar(LCD_Port+2,3);
      for j := 1 to Exectime do  DLPortReadPortUChar($220);
    DlPortWritePortUchar(LCD_Port+2,1);
      for j := 1 to Exectime do  DLPortReadPortUChar($220);
   except else End
  End;
End;

{-------------------------------------------------------}

Procedure LCD_SetIOPropertys(Port : pchar;
                             Exectime : Cardinal;
                             ExectimeGfx : Cardinal;
                             X : Cardinal;
                             Y : Cardinal;
                             gX : Cardinal;
                             gY : Cardinal;
                             LightOn : Boolean;
                             LightSliderValue : Byte;
                             ContrastOn : Boolean;
                             ContrastSliderValue : Byte;
                             Outports : Cardinal;
                             UnderlineMode : Boolean;
                             UnderlineOutput : Boolean);
Begin
LCD_Port:=StrToInt('$'+Port);
LCD_ExectimeGfx := ExectimeGfx;
LCD_Exectime := Exectime;
if LightOn then Light:=240
           else Light:=0;
Ini:=TiniFile.Create(extractfilepath(paramstr(0))+'\controller\OM6211\settings.ini');
 FontType:=Ini.ReadString('init','TextFont','Default');
 CommandModeSet:=Ini.ReadString('init','Text+GraphicsDisplayMode','XOR');
Ini.Free;
if (gX>0)AND(gX<=84) then LCD_Width:=gX else LCD_Width:=84;
if (gY>0)AND(gY<=48) then LCD_Height:=gY else LCD_Height:=48;
if (X>0)AND(X<=14) then LCD_TextCols:=X else LCD_TextCols:=14;
if (Y>0)AND(Y<=6) then LCD_TextRows:=Y else LCD_TextRows:=6;

if ContrastOn then begin
                      Command ($22, LCD_ExectimeGfx);  // Contrast upper byte
                      Command (128+(ContrastSliderValue div 8), LCD_ExectimeGfx) // Contrast lower byte
                   end
              else begin
                      Command ($20, LCD_ExectimeGfx);  // Contrast upper byte
                      Command (128, LCD_ExectimeGfx) ; // Contrast OFF
                   end;

ContrastValue:=ContrastSliderValue;
Contrast_ON:=ContrastOn;
End;

{-------------------------------------------------------}

Procedure LCD_UpdateScreen;   // Procedure to build and send the final picture to LCD
var x,y,Yoffs : integer;

Begin
  LCD_SetOutputAddress(0,0);
  Yoffs := LCD_TextRows;
  for y:=0 to Yoffs-1 do
      Begin
       Command ($B0+y, LCD_ExectimeGfx); // Set Y RAM address
       Command ($10, LCD_ExectimeGfx);   // Set X RAM address
       Command (0, LCD_ExectimeGfx);     // Set X RAM address
       for x:=0 to LCD_Width-1 do
        Begin
         if (CommandModeSet = 'OR')OR(CommandModeSet = 'or') then   // Select merge style
            Data(GraphArray[x,y] OR TextArray[x,y], LCD_ExectimeGfx) else
         if (CommandModeSet = 'AND')OR(CommandModeSet = 'and') then
            Data(GraphArray[x,y] AND TextArray[x,y], LCD_ExectimeGfx) else
            Data(GraphArray[x,y] XOR TextArray[x,y], LCD_ExectimeGfx);
        End;
      End;
//Command (0, LCD_ExectimeGfx);
End;

{-------------------------------------------------------}

Function DLL_GetInfo : TDLLInfo;

var c : integer; x : TDLLInfo;
Begin
     for c:=0 to 255 do x.IDArray[c] := 0;
     for c:=1 to length(ID) do
      Begin
       x.IDArray[c-1] := ord(ID[c]);
      End;

     x.SupportGfxLCD := true;
     x.SupportTxtLCD := true;
     x.SupportLightSlider := false;
     x.SupportContrastSlider := true;
     x.SupportOutports := false;
     x.CCharWidth := 6;
     x.CCharHeight := 8;
     x.FontPitch := 6;

     result := x;
End;

{-------------------------------------------------------}

Function LCD_IsReadyToReceive : boolean;
Begin
result:=true;
End;

{-------------------------------------------------------}

Procedure LCD_SendToController(Value : Byte);
Begin
End;

{-------------------------------------------------------}

Procedure LCD_SEndToGfxMemory(Pixels : TArrayType; X1,Y1,X2,Y2 : Word; inverted : boolean);

Var
  z,Bitmap  : byte;
  i,j,Yoffs : word;

Begin
try
if inverted then Command ($A7, LCD_ExectimeGfx)   // Reverse disp
            else Command ($A6, LCD_ExectimeGfx);  // Normal disp
j:=(Y1 div Font_Height)*Font_Height;
while j<=Y2 do
 Begin
  Yoffs:=j div Font_Height;
  if Yoffs>ScanSize-1 then Yoffs:=ScanSize-1;
   for i := X1 to X2 do
     Begin
      Bitmap := 0;
      for z:= 0 to 7 do
       try
        if Pixels[i+(z*LCD_Width)+(j*LCD_Width)]>0 then Bitmap := Bitmap + 1 shl z;
        GraphArray[i,Yoffs]:=Bitmap;
       except else end;
     End;
   inc(j,Font_Height);
 End;
LCD_UpdateScreen;
except else end;
End;

{-------------------------------------------------------}

Procedure LCD_SendToMemory(Value : Byte);

Var CharCol : integer;
    Data    : byte;
Begin
 try
  if LCD_PosX <= LCD_TextCols-1 then
   Begin
      if (Value >= 0) AND (Value < 32) then Begin  // 6*8 pixels 'CGRAM' characters
        for CharCol := 0 to 5 do
         Begin
          Data := CGchar[Value*6 + CharCol];
          TextArray[LCD_PosX*6+CharCol,LCD_PosY] := Data;
         End;
      end else

      if (Value >= 32) AND (Value < 128) then Begin  // 5*8 pixels 'CGROM' default characters
        for CharCol := 0 to 4 do
         Begin
          if FontType='Cyrillic' then Data := CharTableCyrillic[(Value-32)*5 + CharCol] else
                                      Data := CharTableDefault[(Value-32)*5 + CharCol];
          TextArray[LCD_PosX*6+CharCol,LCD_PosY] := Data;
         End;
        TextArray[LCD_PosX*6+5,LCD_PosY] := $00;  // Space between chars
      end else

      if (Value >= 128) AND (Value < 256) then Begin  // 5*8 pixels 'CGROM' special characters
        for CharCol := 0 to 4 do
         Begin
          Data := CharTableSpecial[(Value-128)*5 + CharCol];
          TextArray[LCD_PosX*6+CharCol,LCD_PosY] := Data;
         End;
        TextArray[LCD_PosX*6+5,LCD_PosY] := $00;  // Space between chars
      End;

    inc(LCD_PosX);
   End;

  if (LCD_PosX > LCD_TextCols-1) AND (LCD_PosY >= LCD_TextRows-1) then
   Begin
    LCD_UpdateScreen;
   End;
 except else End;
End;

{-------------------------------------------------------}

procedure LCD_SetOutputAddress(x,y : Cardinal);
Begin
     if (x <= LCD_TextCols-1) and (y <= LCD_TextRows-1) then
      Begin
       Command ($B0+y, LCD_Exectime); // Ypos
       Command ($10+(x*6) shr 4, LCD_Exectime); // Xpos upper nibble
       Command ((x*6) AND $0F, LCD_Exectime);   // Xpos lower nibble
       LCD_PosX := x;
       LCD_PosY := y;
      End;
End;

{-------------------------------------------------------}

Procedure LCD_SetCGRAMChar(cdata : TCharData);

var
  x,y : byte;
Begin
for x := 0 to 5 do
 Begin
   CGchar[cdata.position*6 + x] := 0;
   for y := 0 to 7 do
     Begin
       if cdata.data[x,y]>0 then
          CGchar[cdata.position*6 + x] := CGchar[cdata.position*6 + x] + 1 shl y;
     End;
 End;
End;

{-------------------------------------------------------}

Function LCD_GetCGRAMChar(position : Byte) : Byte;
Begin result := position;
End;

{-------------------------------------------------------}

Procedure LCD_Init;stdcall;

var m:integer;

Begin
DlPortWritePortUchar(LCD_Port,12);// Reset Low
Sleep(1);
DlPortWritePortUchar(LCD_Port,14);// Reset High
Sleep(1);

Command ($E2, LCD_ExectimeGfx);   // Soft Reset
Command ($28, LCD_ExectimeGfx);   // HV-gen=OFF
Command ($AE, LCD_ExectimeGfx);   // Disp=OFF
Command ($A5, LCD_ExectimeGfx);   // All=ON PWR down

if Contrast_ON then begin
                      Command ($22, LCD_ExectimeGfx);  // Contrast upper byte
                      Command (128+(ContrastValue div 8), LCD_ExectimeGfx) // Contrast lower byte
                   end
              else begin
                      Command ($20, LCD_ExectimeGfx);  // Contrast upper byte
                      Command (128, LCD_ExectimeGfx) ; // Contrast OFF
                   end;
Command ($38, LCD_ExectimeGfx);   // TEMP=0
Command ($34, LCD_ExectimeGfx);   // BIAS=4
Command ($3E, LCD_ExectimeGfx);   // HV=2
Command ($A4, LCD_ExectimeGfx);   // All=OFF
sleep(2);
Command ($B0, LCD_ExectimeGfx);   // Y=0
Command ($10, LCD_ExectimeGfx);   // X=0
Command (0, LCD_ExectimeGfx);     // X=0
Command ($40, LCD_ExectimeGfx);   // Start row=0
Command ($2F, LCD_ExectimeGfx);   // HVgen=ON

for m := 0 to 503{6x84-1}do Data (0, LCD_ExectimeGfx); // Clear display RAM
Command ($AF, LCD_ExectimeGfx);   // Disp ON

if Once then
Begin
if (LCD_Width = 84)AND(LCD_Height = 48)then Logo84x48 else  // BootLogo
                                            Command ($A5, LCD_ExectimeGfx);   // All=ON
Sleep(1000);
Command ($A4, LCD_ExectimeGfx); // All=OFF
Command ($B0, LCD_ExectimeGfx); // Y=0
Command ($10, LCD_ExectimeGfx); // X=0
Command (0, LCD_ExectimeGfx);   // X=0
for m := 0 to 503{6x84-1}do Data (0, LCD_ExectimeGfx); // Clear display RAM
Once:=false;
End;
End;

{-------------------------------------------------------}

Procedure LCD_CleanUp;

var m:integer;
Begin
for m := 0 to 503{6x84-1}do Data (0, LCD_ExectimeGfx);
Command ($AE, LCD_ExectimeGfx); // Disp=OFF
Command ($A5, LCD_ExectimeGfx); // All=ON PWR down
Command ($2E, LCD_ExectimeGfx); // HVgen=OFF
Command ($80, LCD_ExectimeGfx); // Vpr=0
Command ($20, LCD_ExectimeGfx); // Vpr=0
//DlPortWritePortUchar(LCD_Port,14); {Lights OFF}
End;

{-------------------------------------------------------}

Procedure Logo84x48;

Const LogoData : array [1..504] of byte =(
$ff, $ff, $ff, $ff, $ff, $ff, $ff, $ff, $1f, $ef, $f7, $f7, $f7, $ef, $1f, $ff, $f7,
$7,  $c7, $3f, $ff, $3f, $c7, $7,  $f7, $ff, $3f, $4f, $af, $b7, $77, $ef, $f7, $f7,
$77, $8f, $ff, $f7, $7,  $ff, $ff, $ff, $f7, $7,  $ff, $ff, $ff, $ff, $ff, $ff, $7f,
$bf, $b7, $7,  $ff, $bf, $3f, $7f, $bf, $bf, $37, $ff, $bf, $3f, $ff, $bf, $3f, $3f,
$bf, $bf, $7f, $ff, $bf, $3f, $7f, $bf, $ff, $ff, $ff, $ff, $ff, $ff, $ff, $ff,
$ff, $ff, $ff, $ff, $ff, $ff, $ff, $ff, $fe, $fd, $fb, $fb, $fb, $fd, $fe, $ff, $fb,
$f8, $fb, $fe, $f9, $fe, $fb, $f8, $fb, $ff, $fc, $fb, $fb, $fb, $fc, $fb, $f9, $fa,
$fb, $f9, $ff, $fb, $f8, $fb, $ff, $ff, $fb, $f8, $fb, $ff, $ff, $ff, $ff, $ff, $fc,
$fb, $fb, $f8, $fb, $fb, $f8, $fb, $ff, $fb, $f8, $fb, $ff, $fe, $f9, $fe, $ff, $fc,
$fa, $fa, $fa, $ff, $fb, $f8, $fb, $ff, $ff, $ff, $ff, $ff, $ff, $ff, $ff, $ff,
$ff, $ff, $ff, $ff, $1b, $1b, $1f, $ff, $fb, $fb, $ff, $ff, $fb, $fb, $ff, $ff, $1b,
$9b, $9f, $9f, $9b, $9b, $9f, $9f, $fb, $fb, $ff, $1f, $1b, $9b, $9f, $9f, $9b, $9b,
$1f, $1f, $fb, $ff, $83, $e7, $e7, $e7, $e7, $e7, $e7, $7,  $7,  $ff, $ff, $3,  $3,
$7f, $7f, $7f, $7f, $7f, $3,  $3,  $ff, $ff, $1b, $1b, $9b, $9b, $9b, $9b, $9b, $83,
$83, $ff, $ff, $1b, $1b, $db, $db, $db, $db, $db, $db, $db, $ff, $ff, $ff, $ff,
$ff, $ff, $ff, $ff, $c0, $c0, $c0, $cf, $cf, $cf, $cf, $cf, $cf, $cf, $ff, $ff, $c0,
$cf, $cf, $cf, $cf, $ff, $ff, $cf, $ff, $ff, $ff, $c0, $c0, $cf, $cf, $cf, $ff, $ff,
$c0, $c0, $ff, $ff, $d9, $df, $ff, $ff, $df, $df, $ff, $fc, $dc, $df, $ff, $ff, $df,
$df, $ff, $ff, $df, $df, $fc, $fc, $df, $df, $fc, $fc, $df, $df, $ff, $ff, $df, $df,
$ff, $ff, $df, $dc, $fc, $fc, $dc, $dc, $fc, $fc, $dc, $dc, $ff, $ff, $ff, $ff,
$ff, $7f, $ff, $ff, $ff, $ff, $ff, $ff, $ff, $ff, $ff, $ff, $ff, $ff, $7f, $7f, $7f,
$ff, $ff, $ff, $ff, $ff, $ff, $ff, $ff, $ff, $ff, $ff, $ff, $ff, $ff, $ff, $ff, $ff,
$ff, $ff, $ff, $7f, $7f, $ff, $3f, $ff, $7f, $ff, $7f, $ff, $ff, $ff, $ff, $ff, $7f,
$7f, $ff, $ff, $ff, $ff, $ff, $ff, $ff, $ff, $ff, $ff, $ff, $7f, $ff, $ff, $ff, $ff,
$ff, $ff, $ff, $ff, $ff, $7f, $ff, $ff, $ff, $ff, $ff, $ff, $ff, $ff, $ff, $ff,
$ff, $c0, $dd, $dd, $e3, $ff, $fd, $63, $9f, $e3, $fd, $ff, $ff, $e0, $df, $df, $df,
$e0, $ff, $c1, $fd, $fd, $e3, $dd, $dd, $e3, $ff, $db, $d5, $d5, $ed, $cd, $d5, $d9,
$ff, $fd, $c0, $cd, $d5, $d5, $c1, $7f, $81, $ff, $c1, $ff, $ff, $ff, $ff, $ec, $db,
$db, $e6, $ff, $cd, $d5, $d9, $ff, $cd, $d5, $d5, $c1, $ff, $c0, $dd, $dd, $e3, $ff,
$e3, $dd, $dd, $e3, $ff, $c0, $ff, $e3, $dd, $dd, $eb, $db, $d5, $d5, $ed, $ff);

Var
 c : Word;
Begin
Command ($B0, LCD_ExectimeGfx);   // Y=0
Command ($10, LCD_ExectimeGfx);   // X=0
Command (0, LCD_ExectimeGfx);     // X=0
for c:=1 to SizeOf(LogoData) do
 Begin
   Data (LogoData[c], LCD_ExectimeGfx);
 End;
End;

{-------------------------------------------------------}

Procedure LCD_Customize(SymData : TSymbolInfo);
Begin
End;

{-------------------------------------------------------}

End.

